const path = require('path')
const fs = require('fs')
const program = require('commander')
const inquirer = require('inquirer')
const shell = require('shelljs')
const ora = require('ora')
const colors = require('colors')
const generator = require('../templates/generator')
const download = require('../lib/download')

let exec = require('child_process').exec
// 运行环境path
let processPath = process.cwd()
colors.setTheme({
	silly: 'rainbow',
	input: 'grey',
	verbose: 'cyan',
	prompt: 'grey',
	info: 'green',
	data: 'grey',
	help: 'cyan',
	warn: 'yellow',
	debug: 'blue',
	error: 'red'
})

// 判断文件夹不存在
function dirIsNull (dirPath) {
	let dirInfo = shell.ls('-A', dirPath)
	return dirInfo.stdout == ''
}

program
	.version('1.0.0')
	.usage('<project-name>')
	.parse(process.argv)

// 根据输入，获取项目名称
let projectName = program.args[0]

// 根据用户数据合并目标构建文件夹
let rootPath = path.resolve(processPath, projectName || '')

// 判断目标文件夹是否存在
new Promise((resolve, reject) => {
	if (fs.existsSync(rootPath)) {
		console.log(rootPath)
		
		// 如果文件夹非空
		if (!dirIsNull(rootPath)) {
			console.log(`文件夹非空：${processPath}`.warn)
		}
		
		inquirer
			.prompt([{
				name: 'init',
				message: `确定在 ${rootPath} 目录下生成项目？ Y/N`,
				default: 'Y'
			}
			])
			.then(answers => {
				if (answers.init.toUpperCase() == 'Y') {
					resolve()
				} else {
					console.log('初始化项目失败'.error)
				}
			})
	} else {
		fs.mkdir(rootPath, () => {
			resolve()
		})
	}
}).then(() => {
	getProjetInfo()
})

function getProjetInfo () {
	console.log('开始构建项目'.info)
	
	download(rootPath)
		.then(target => {
			console.log(`======${target}=======`)
			
			let context = {
				name: projectName || '项目名称'
			}
			inquirer
				.prompt([
					{
						name: 'projectName',
						message: '项目的名称',
						default: context.name
					}, {
						name: 'projectVersion',
						message: '项目的版本号',
						default: '1.0.0'
					}, {
						name: 'projectDescription',
						message: '项目的简介',
						default: `A project named `
					}, {
						name: 'projectauthor',
						message: '项目的作者',
						default: `A project named`
					}
				])
				.then(answers => {
					// 用户输入自定义项赋值给context传入package.json
					context = answers
					generator(context, rootPath)
					shell.rm('-rf', path.resolve(rootPath, '.git'))
					console.log(`项目构建成功:${rootPath}`.info)
					askNpm()
				})
		})
		.catch(err => console.log(err))
}

function askNpm () {
	let npmChoice = ['使用 yarn 安装', '使用 cnpm 安装', '使用 npm 安装', '稍后自己安装']
	
	inquirer
		.prompt([
			{
				type: 'list',
				name: 'npm',
				message: '安装npm依赖包?',
				choices: npmChoice,
				default: 0
			}
		])
		.then(answers => {
			let npmIndex = npmChoice.indexOf(answers.npm)
			let shellStr = ''
			switch (npmIndex) {
				case 0: shellStr = 'cnpm install'
					break
				case 1: shellStr = 'cnpm install'
					break
				case 2: shellStr = 'npm install'
					break
			}
			const spinner = ora(`正在安装NPM依赖`)
			spinner.start()
			
			if (shellStr != '') {
				exec(shellStr, (err, stdout, stderr) => {
					if (err) {
						spinner.fail()
						console.log(stderr.error)
					} else {
						spinner.succeed()
						console.log('NPM安装完成'.info)
					}
				})
			}
		})
}
